$GEN
$SAVE
$NOLIST
;*****************************************************************************
; File -- common.inc
; Purpose -- Include file for COS DCE operate code.  Contains equates and
;            MACRO definitions.
; Author -- William I. Clement, Ph.D., President
;           Clement Engineering, Inc.
;           1261 Dogwood Road
;           Arnold, MD  21012-2220
;           (410) 518-6591
;
; Change History --
; 23Oct00   First Operate-mode version based on Boot-mode verion 01.05
;*****************************************************************************

;*****************************************************************************
;              Segment addresses for the assemble/link process
;*****************************************************************************
PATCHCST_SEG   equ   0D000h   ; location of patchable constants in CODE space
HISTORY_SEG    equ   0E000h   ; History buffer segment (XDATA)
SCRATCH_SEG    equ   0FF00h   ; scratch space (XDATA)

TESTCODE_SEG   equ   1000h    ; Some test code is placed well outside the
                              ; operate code range (so it will not be in the
                              ; image).  This entire segment is only included
                              ; in a build if the line
                              ;   $SET (INCLUDE_TEST_CODE=1)
                              ; is included at the top of file opermain.a51
;-----------------------------------------------------------------------------

;*****************************************************************************
;                        Operate Mode Diagnostic Codes
;*****************************************************************************
DIAG0001     equ     5A01H    ; Bad upload length
DIAG0002     equ     5A02H    ; Bad upload CRC
DIAG0003     equ     5A03H    ; Bad download length
DIAG0004     equ     5A04H    ; Wrong command MSB, should be 1
DIAG0005     equ     5A05H    ; Command byte must have twin
DIAG0006     equ     5A06H    ; Command Parameters must have complements
DIAG0011     equ     5A11H    ; Illegal command
DIAG0013     equ     5A13H    ; Command did not complete execution
DIAG0017     equ     5A17H    ; Stack Alarm (stack growing)
DIAG001F     equ     5A1FH    ; Commanded reset failure
DIAG0031     equ     5A31H    ; Attempt to execute nonexistent ISR
DIAG0032     equ     5A32h    ; Background CRC error

; Operate mode (only) diagnostics ...
DIAG0008     equ     5A08H    ; HV not enabled. LFHVENA command is required.
DIAG000B     equ     5A0BH    ; Motor not enabled.  LFRMENA command required.
DIAG0010     equ     5A10h    ; Door actuators not enabled.  LFRACTEN req.
DIAG0015     equ     5A15h    ; Actuator already latched, LFRACTRS command ignored.
DIAG0016     equ     5A16h    ; AUX Current exceeded LFRILIM during HV Operations.
DIAG0022     equ     5A22h    ; CRP shutdown due to excessive counts in the Fast Event counter (FEC). 
DIAG0026     equ     5A26h    ; AUX current exceeded door current limit. Door motor and actuators safed. 
DIAG0027     equ     5A27h    ; HV-A current exceeded limit ONE time
DIAG0028     equ     5A28h    ; HV-B current exceeded limit ONE time
DIAG002A     equ     5A2Ah    ; HV-A current exceeded limit past persistence
DIAG002B     equ     5A2Bh    ; HV-B current exceeded limit past persistence
DIAG002E     equ     5A2Eh    ; Aux current exceeded limit past persistence
DIAG0033     equ     5A33h    ; Attempt to set VTGT > VMAX, Segment A
DIAG0034     equ     5A34h    ; Attempt to set VMAX < VTGT, Segment A
DIAG0035     equ     5A35h    ; Attempt to set VTGT > VMAX, Segment B
DIAG0036     equ     5A36h    ; Attempt to set VMAX < VTGT, Segment B
;-----------------------------------------------------------------------------

;*****************************************************************************
;              Memory Mapped Registers (with read/write usage)
;*****************************************************************************
LED          equ     2000h    ;   W, LEDs
MUX          equ     4000h    ;   W, MUX select input to ADC
HV_STATUS    equ     4000h    ; R  , Door status (end switches and latch)
CONTROLS1    equ     4200h    ; R/W, Board C
CONTROLS2    equ     4300h    ; R/W, Board C, UNLOCK required
VMAXA        equ     4400H    ;   W, HV DAC A Max (pin 2)
VSETA        equ     4401H    ;   W, HV DAC A Setting (pin 1)
VMAXB        equ     4402H    ;   W, HV DAC B Max (pin 24)
VSETB        equ     4403H    ;   W, HV DAC B Setting (pin 23)
BILEVEL      equ     4500h    ; R  , Bilevel Status Register
ADC_SPACE    equ     4700h    ; R/W, ADC (write to start, read value)
RXD_HARDWARE equ     5000h    ; R  , read from CS computer
TXD_HARDWARE equ     5000h    ;   W, write to CS computer
COUNTERS_A   equ     6000h    ; R  , Digitizer A counters
FEC_A        equ     6000h    ; R  , Digitizer A FEC (24 bits)
DEC_A        equ     6004h    ; R  , Digitizer A DEC (24 bits)
SDC_1        equ     6008h    ; R  , SDC 1 (24 bits)
COUNTERS_B   equ     6100h    ; R  , Digitizer B counters
FEC_B        equ     6100h    ; R  , Digitizer B FEC (24 bits)
DEC_B        equ     6104h    ; R  , Digitizer B DEC (24 bits)
SDC_2        equ     6108h    ; R  , SDC 2 (24 bits)
PH_A         equ     6200h    ; R  , Pulse Height Histogram Side A
PH_B         equ     6300h    ; R  , Pulse Height Histogram Side B

; MUX addresses for the three current measurements ...
LVPS_AUX_MON  equ    21h      ; Auxillary power supply
HV_IMON_A     equ    22h      ; HV-A current monitor
HV_IMON_B     equ    23h      ; HV-B current monitor
;-----------------------------------------------------------------------------

;*****************************************************************************
;                        Housekeeping Script Opcodes
;*****************************************************************************
op_end       equ     00h      ; end of housekeeping script
op_code      equ     01h      ; copy from CODE space into housekeeping packet
op_xdata     equ     02h      ; copy from XDATA space (external RAM)
op_data      equ     03h      ; copy from DATA space (internal RAM)
op_a2d       equ     06h      ; read ADC analog (8-bit)
op_jump      equ     07h      ; jump to specified address in HKP packet
op_skip      equ     08h      ; skip 'N' bytes ahead in housekeeping packet
op_clear     equ     09h      ; clear 'N' bytes of housekeeping packet
op_TDC_A     equ     46h      ; read analog from TDC 'A'
op_TDC_B     equ     82h      ; read analog from TDC 'B'
;-----------------------------------------------------------------------------

;*****************************************************************************
;                       Command and Upload Data Buffers
;*****************************************************************************
RS00         equ     0000h       ; Operate mode base receive address
RS01         equ     RS00+0040h  ; Upload data (1024 bytes)
RS17         equ     RS00+0440h  ; Command Packet (28 bytes)
SERIALN      equ     RS17+4      ; Command Serial Number
PARAM1       equ     RS17+8      ; Command Parameter 1
PARAM2       equ     RS17+12     ; Command Parameter 2
PARAM3       equ     RS17+16     ; Command Parameter 3
PARAM4       equ     RS17+20     ; Command Parameter 4
PARAM5       equ     RS17+24     ; Command Parameter 5
;-----------------------------------------------------------------------------

;*****************************************************************************
;                   Housekeeping and Download Data Buffers
;*****************************************************************************
TS00         equ     1000h       ; Operage mode base transmit address
TS01         equ     TS00+0040h  ; Download data (1024 bytes)
TS17         equ     TS00+0440h  ; Housekeeping packet (1024 bytes)
;-----------------------------------------------------------------------------

;*****************************************************************************
;                       8051 Output Pins and Mnemonics
;*****************************************************************************
; TDC 3-wire interface ...
TDC_CLOCK    equ     P1.0   ; data clocked out low-to-high transition
                            ; (keep high for 1.5 us, used for both TDCs)
TDC_DATA     equ     P1.1   ; the data bit (used for both TDCs)
TDC_A_ENABLE equ     P1.2   ; 1 = TDC 'A' enabled
TDC_B_ENABLE equ     P1.3   ; 1 = TDC 'B' enabled
; Hardware Lock/Unlock interface ...
C2_LOCK      equ     P1.4   ; 1 = CONTROLS2 is locked
PHA_DIS      equ     P1.5   ; 1 = ACTEL disabled so 8051 may read PH data
                            ; 0 = ACTEL collecting and storing PH data
RAM_BANK     equ     P1.6   ; 1 = DCE-B RAM chip U8 address 0000-3FFFh
                            ; 0 = DCE-B RAM chip U8 address 4000-7FFFh
; Various reset lines ...
ACTEL_RESET  equ     P1.7   ; 0 = Reset Counter Actel (10us minimum low pulse)
                            ; 1 = Counter Actels running
HARDRESET    equ     P3.5   ; RESET control line for 8051 plus ACTEL 'CPU'
;-----------------------------------------------------------------------------

;*****************************************************************************
;                    8051 Programmable Counter Array (PCA)
;*****************************************************************************
CL           equ     0E9h   ; PCA Counter LSB
CH           equ     0F9h   ; PCA Counter MSB
CCON         equ     0D8h   ; PCA timer enable
CMOD         equ     0D9h   ; PCA clock source
CCAPM4       equ     0DEh   ; CCAPM4 setup as WatchDog Timer
CR           equ     0DEh   ; Bit in CCON
CCAP4L       equ     0EEh   ; watchdog comparitor to PCA Counter LSB
CCAP4H       equ     0FEh   ; watchdog comparitor to PCA Counter MSB
;-----------------------------------------------------------------------------

$SET (GOTO_WEEDS=1)

;*****************************************************************************
;                                   Macros
;*****************************************************************************
; "FILL_TO_BOUNDARY" pads code space up to the next "BOUND"-byte boundary
; with argument X.  "BOUND" must be a power of 2.
$if GOTO_WEEDS=1

FILL_TO_BOUNDARY MACRO   BOUND, X
$SAVE
$NOLIST
check       equ  (BOUND AND (BOUND-1))
if check <> 0
   $RESTORE
   EXITM        ; BOUND wasn't a power of 2
endif
mylocation  equ  $
sizetofill  equ  (((mylocation+BOUND-1) AND (10000h-BOUND)) - mylocation)

if sizetofill = 0
   $RESTORE
   EXITM
endif

if sizetofill < 3
   $RESTORE
   REPT   sizetofill
   DB     X
   ENDM
   EXITM
endif

if sizetofill = 3
$RESTORE
   lcall In_The_Weeds
   EXITM
endif

sizetofill  equ   sizetofill-3
$RESTORE
   REPT   sizetofill
   DB     X
   ENDM
   lcall In_The_Weeds
ENDM                        ; FILL_TO_BOUNDARY

$else                       ; else don't GOTO_WEEDS

FILL_TO_BOUNDARY MACRO   BOUND, X
$SAVE
$NOLIST
check       equ  (BOUND AND (BOUND-1))
if check <> 0
   $RESTORE
   EXITM        ; BOUND wasn't a power of 2
endif
mylocation  equ  $
sizetofill  equ  (((mylocation+BOUND-1) AND (10000h-BOUND)) - mylocation)
if sizetofill = 0
   $RESTORE
   EXITM
endif
$RESTORE
   REPT   sizetofill
   DB     X
   ENDM
ENDM                        ; FILL_TO_BOUNDARY

$endif

;-----------------------------------------------------------------------------
; "FILL_TO_ADDRESS_CRC" pads code space up to "ADDRESS" with argument X.  Last
; two bytes are occupied by "DELTA_CRC".  If "DELTA_CRC" is not supplied, then
; last two bytes also contain argument X.
$if GOTO_WEEDS=1

FILL_TO_ADDRESS_CRC MACRO   ADDRESS, X, DELTA_CRC
$SAVE
$NOLIST
mylocation  equ  $
if ADDRESS<=(mylocation+5)
   $RESTORE
   EXITM
endif
$RESTORE
sizetofill  equ  (ADDRESS-mylocation-5)
$SAVE
$NOLIST
   REPT   sizetofill
   DB     X
   ENDM
$RESTORE
   lcall  In_The_Weeds
if NUL  DELTA_CRC
   DB   X, X
else
   DW   DELTA_CRC
endif
ENDM                        ; FILL_TO_ADDRESS_CRC

$else                       ; else don't GOTO_WEEDS

FILL_TO_ADDRESS_CRC MACRO   ADDRESS, X, DELTA_CRC
$SAVE
$NOLIST
mylocation  equ  $
if ADDRESS<=(mylocation+2)
   $RESTORE
   EXITM
endif
$RESTORE
sizetofill  equ  (ADDRESS-mylocation-2)
$SAVE
$NOLIST
   REPT   sizetofill
   DB     X
   ENDM
$RESTORE
if NUL  DELTA_CRC
   DB   X, X
else
   DW   DELTA_CRC
endif
ENDM                        ; FILL_TO_ADDRESS_CRC

$endif

;-----------------------------------------------------------------------------
; "ISR_SAVE_STATE" pushes several registers on the stack for context saving.
ISR_SAVE_STATE MACRO
   push  ACC
   push  B
   push  DPH
   push  DPL
   push  PSW
ENDM                        ; ISR_SAVE_STATE
;-----------------------------------------------------------------------------
; "ISR_RESTORE_STATE" pops the registers saved by "ISR_SAVE_STATE" macro.
ISR_RESTORE_STATE MACRO
   pop   PSW
   pop   DPL
   pop   DPH
   pop   B
   pop   ACC
ENDM                        ; ISR_RESTORE_STATE
;-----------------------------------------------------------------------------
NOP6  MACRO                 ; pad code with 6 nop() opcodes for patch jumps
   nop
   nop
   nop
   nop
   nop
   nop
ENDM
;-----------------------------------------------------------------------------
; "PUSH_REGS" saves current register set.
PUSH_REGS MACRO
   mov   REGS, R0
   push  REGS
   mov   REGS, R1
   push  REGS
   mov   REGS, R2
   push  REGS
   mov   REGS, R3
   push  REGS
   mov   REGS, R4
   push  REGS
   mov   REGS, R5
   push  REGS
   mov   REGS, R6
   push  REGS
   mov   REGS, R7
   push  REGS
ENDM                        ; PUSH_REGS
;-----------------------------------------------------------------------------
; "POP_REGS" restores the register set saved by "PUSH_REGS" macro.
POP_REGS MACRO
   pop   REGS
   mov   R7, REGS
   pop   REGS
   mov   R6, REGS
   pop   REGS
   mov   R5, REGS
   pop   REGS
   mov   R4, REGS
   pop   REGS
   mov   R3, REGS
   pop   REGS
   mov   R2, REGS
   pop   REGS
   mov   R1, REGS
   pop   REGS
   mov   R0, REGS
ENDM                        ; POP_REGS
;-----------------------------------------------------------------------------
; Macro 'SET_STATUS' sets the destination bit 'dest' if source bit 'src' is
; set.  It DOES NOT clear 'dest' if 'set' is clear.  This macro is included
; so that the status words can be created without the need for many global
; labels.
SET_STATUS MACRO  dest, src
   local  skip
   jnb   src, skip
   setb  dest
skip:
ENDM
;-----------------------------------------------------------------------------
; Macro 'CALL_DPTR' allows one to make a CALL to a subroutine pointed to by
; DPTR.  This is an extension of the 8051's capabilities which allow a JUMP to
; such a location.  Here we push the return address on the stack manually
; and then make a regular 'JMP @A+DPTR' call.
CALL_DPTR MACRO
   local  return_location
   ; save ACC on stack
   push  ACC
   ; Push the address of label 'return_location' onto stack as return address.
   mov   A, #LOW(return_location)
   push  ACC
   mov   A, #HIGH(return_location)
   push  ACC
   clr   A                  ; prepare for code jump
   jmp   @A+DPTR            ; jump to location pointed to by DPTR.  The 'ret'
                            ; at the end of the function will return to the
                            ; label 'return_location'.
return_location:
   pop   ACC                ; restore ACC
ENDM
;-----------------------------------------------------------------------------

;*****************************************************************************
;                  PUBLIC and EXTRN Declarations (by module)
;*****************************************************************************
; Franklin Assembler/Linker places global variable _STACK at the base of
; the internal memory used.  Assigning the Stack Pointer (SP) this value
; will maximize the internal stack space available to the program.
EXTRN IDATA  (_STACK)
;-----------------------------------------------------------------------------
$if (OPERATE_MAIN)
PUBLIC        maxSP, DOOR_TIMER
PUBLIC        reg3, reg4, reg5, reg6, reg7, reg8, reg9
PUBLIC        reg3len, reg4len, reg5len, reg6len, reg7len, reg8len, reg9len
PUBLIC        In_The_Weeds
$else
EXTRN  DATA   (maxSP, DOOR_TIMER)
EXTRN  NUMBER (reg3, reg4, reg5, reg6, reg7, reg8, reg9)
EXTRN  NUMBER (reg3len, reg4len, reg5len, reg6len, reg7len, reg8len, reg9len)
EXTRN  CODE   (In_The_Weeds)
$endif

$if (PATCHCST)
PUBLIC        ilim_param_hva, ilim_param_hvb, ilim_param_aux, ilim_param_all
PUBLIC        crp_segA, crp_segB, hv_segA, hv_segB, hv_segAB
PUBLIC        door_task_params
PUBLIC        mLFSBITS1, mLFSBITS2, mLFDPERF, mLFDLSTC
PUBLIC        mLFDMEXTR, mLFDMDATA, mLFDMONS, mLFDMADD
PUBLIC        mLFDDIAGS, mDIAG_NUM, mLFDERR, mLFDERRP, mLFDERRcpy
PUBLIC        mLFCPKTL, mLFDCMDR, mLFDCMDX, mLFMXFER
PUBLIC        mLFCTIME
PUBLIC        mCWNS, mWNS, mTICKS, mLFDDETID
PUBLIC        mLFSBITS0, mLFSBITS1, mLFSBITS2
PUBLIC        mLFSBITS3, mLFSBITS4, mLFSBITS5
PUBLIC        mCURR_5, mCURR_NUM
PUBLIC        HSTERR_TABLE, hkp_script
PUBLIC        mLFMCRC, mLFMROM, CRC_REGIONS
PUBLIC        sz_regions, num_regions
PUBLIC        mLFGSTIMA, mLFGSTIMB
PUBLIC        mLFHVILIM
PUBLIC        start_history_seg, length_history_seg
PUBLIC        start_scratch_seg, length_scratch_seg
PUBLIC        mFEC_A, mDEC_A, mSDC_1, mFEC_B, mDEC_B, mSDC_2
PUBLIC        mBADISR, mBADREGION, mBADCRP, mBADAUXI, mPATTERN, mVERR
PUBLIC        mADC_DELAY, mMUX_DELAY, DOOR_TIMEOUT
PUBLIC        aLFTDVAA
PUBLIC        aLFGSHFAX
PUBLIC        aLFGSHFBX
PUBLIC        mElapsed
PUBLIC        mCRP_HISTORY, mWeeds
$else
EXTRN  CODE   (ilim_param_hva, ilim_param_hvb, ilim_param_aux, ilim_param_all)
EXTRN  CODE   (crp_segA, crp_segB, hv_segA, hv_segB, hv_segAB)
EXTRN  CODE   (door_task_params)
EXTRN  XDATA  (mLFSBITS1, mLFSBITS2, mLFDPERF, mLFDLSTC)
EXTRN  XDATA  (mLFDMEXTR, mLFDMDATA, mLFDMONS, mLFDMADD)
EXTRN  XDATA  (mLFDDIAGS, mDIAG_NUM, mLFDERR, mLFDERRP, mLFDERRcpy)
EXTRN  XDATA  (mLFCPKTL, mLFDCMDR, mLFDCMDX, mLFMXFER)
EXTRN  XDATA  (mLFCTIME)
EXTRN  XDATA  (mCWNS, mWNS, mTICKS, mLFDDETID)
EXTRN  XDATA  (mLFSBITS0, mLFSBITS1, mLFSBITS2)
EXTRN  XDATA  (mLFSBITS3, mLFSBITS4, mLFSBITS5)
EXTRN  XDATA  (mCURR_5, mCURR_NUM)
EXTRN  XDATA  (mLFGSTIMA, mLFGSTIMB)
EXTRN  CODE   (mLFHVILIM)
EXTRN  CODE   (HSTERR_TABLE, hkp_script)
EXTRN  CODE   (mLFMCRC, mLFMROM, CRC_REGIONS)
EXTRN  NUMBER (sz_regions, num_regions)
EXTRN  NUMBER (start_history_seg, length_history_seg)
EXTRN  NUMBER (start_scratch_seg, length_scratch_seg)
EXTRN  XDATA  (mFEC_A, mDEC_A, mSDC_1, mFEC_B, mDEC_B, mSDC_2)
EXTRN  XDATA  (mBADISR, mBADREGION, mBADCRP, mBADAUXI, mPATTERN, mVERR)
EXTRN  CODE   (mADC_DELAY, mMUX_DELAY, DOOR_TIMEOUT)
EXTRN  NUMBER (aLFTDVAA)
EXTRN  NUMBER (aLFGSHFAX)
EXTRN  NUMBER (aLFGSHFBX)
EXTRN  XDATA  (mElapsed)
EXTRN  XDATA  (mCRP_HISTORY, mWeeds)
$endif
;-----------------------------------------------------------------------------

$RESTORE

